<?php
/*
Plugin Name: SEO Tag Cloud Widget
Plugin URI: http://blog.fleischer.hu/wordpress/seo-tag-cloud/
Description: SEO Tag Cloud Widget displays the tag cloud in a SEO-friendly way, using a search engine optimized html markup.
Version: 1.5.3
Author: Gavriel Fleischer
Author URI: http://blog.fleischer.hu/gavriel/
*/

// Multi-language support
if (defined('WPLANG') && function_exists('load_plugin_textdomain')) {
	load_plugin_textdomain('seo-tag-cloud', PLUGINDIR.'/'.dirname(plugin_basename(__FILE__)).'/lang', dirname(plugin_basename(__FILE__)).'/lang');
}

/**
 * Display tag cloud.
 *
 * The text size is set by the 'smallest' and 'largest' arguments, which will
 * use the 'unit' argument value for the CSS text size unit. The 'format'
 * argument can be 'flat' (default), 'list', 'nolink', or 'array'. The flat value for the
 * 'format' argument will separate tags with spaces. The list value for the
 * 'format' argument will format the tags in a UL HTML list. The nolink value for the
 * 'format' argument will display the tags without links. The array value for the
 * 'format' argument will return in PHP array type format.
 *
 * The 'orderby' argument will accept 'name' or 'count' and defaults to 'name'.
 * The 'order' is the direction to sort, defaults to 'ASC' and can be 'DESC'.
 *
 * The 'number' argument is how many tags to return. By default, the limit will
 * be to return the top 20 tags in the tag cloud list.
 *
 * The 'topic_count_text_callback' argument is a function, which, given the count
 * of the posts  with that tag, returns a text for the tooltip of the tag link.
 * @see default_topic_count_text
 *
 * The 'exclude' and 'include' arguments are used for the {@link get_tags()}
 * function. Only one should be used, because only one will be used and the
 * other ignored, if they are both set.
 *
 * @since 2.3.0
 *
 * @param array|string $args Optional. Override default arguments.
 * @return array Generated tag cloud, only if no failures and 'array' is set for the 'format' argument.
 */
function seo_tag_cloud( $args = '' ) {
	$defaults = array(
		'largest' => 10, 'number' => 20,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'exclude' => '', 'include' => '', 'link' => 'view', 'target' => '',
	);
	$args = wp_parse_args( $args, $defaults );

	$tags = get_tags( array_merge( $args, array( 'orderby' => 'count', 'order' => 'DESC' ) ) ); // Always query top tags

	if ( empty( $tags ) )
		return;

	foreach ( $tags as $key => $tag ) {
		if ( 'edit' == $args['link'] )
			$link = get_edit_tag_link( $tag->term_id );
		else
			$link = get_tag_link( $tag->term_id );
		if ( is_wp_error( $link ) )
			return false;

		$tags[ $key ]->link = $link;
		$tags[ $key ]->id = $tag->term_id;
	}

	$return = seo_tag_cloud_generate( $tags, $args ); // Here's where those top tags get sorted according to $args

	$return = apply_filters( 'wp_tag_cloud', $return, $args );

	if ( 'array' == $args['format'] )
		return $return;

	echo $return;
}

/**
 * Generates a tag cloud (heatmap) from provided data.
 *
 * The text size is set by the 'smallest' and 'largest' arguments, which will
 * use the 'unit' argument value for the CSS text size unit. The 'format'
 * argument can be 'flat' (default), 'list', 'nolink' or 'array'. The flat value for the
 * 'format' argument will separate tags with spaces. The list value for the
 * 'format' argument will format the tags in a UL HTML list. The nolink value for the
 * 'format' argument will display the tags without links. The array value for the
 * 'format' argument will return in PHP array type format.
 *
 * The 'orderby' argument will accept 'name' or 'count' and defaults to 'name'.
 * The 'order' is the direction to sort, defaults to 'ASC' and can be 'DESC' or
 * 'RAND'.
 *
 * The 'number' argument is how many tags to return. By default, the limit will
 * be to return the entire tag cloud list.
 *
 * The 'topic_count_text_callback' argument is a function, which given the count
 * of the posts  with that tag returns a text for the tooltip of the tag link.
 * @see default_topic_count_text
 *
 *
 * @todo Complete functionality.
 * @since 2.3.0
 *
 * @param array $tags List of tags.
 * @param string|array $args Optional, override default arguments.
 * @return string
 */
function seo_tag_cloud_generate( $tags, $args = '' ) {
	global $wp_rewrite;
	$defaults = array(
		'largest' => 10, 'number' => 0,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'topic_count_text_callback' => 'seo_tag_cloud_default_topic_count_text',
		'target' => ''
	);

	if ( !isset( $args['topic_count_text_callback'] ) && isset( $args['single_text'] ) && isset( $args['multiple_text'] ) ) {
		$body = 'return sprintf (
			__ngettext('.var_export($args['single_text'], true).', '.var_export($args['multiple_text'], true).', $count),
			number_format_i18n( $count ));';
		$args['topic_count_text_callback'] = create_function('$count', $body);
	}

	$args = wp_parse_args( $args, $defaults );

	extract( $args );

	if ( empty( $tags ) )
		return;

	// SQL cannot save you; this is a second (potentially different) sort on a subset of data.
	uasort( $tags, create_function('$a, $b', 'return ($a->count < $b->count);') );
	if ( $number > 0 )
		$tags = array_slice($tags, 0, $number);
	if ('posts' != $orderby) {
		uasort( $tags, create_function('$a, $b', 'return strnatcasecmp($a->'.$orderby.', $b->'.$orderby.');') );
	}
	if ( 'DESC' == $order )
		$tags = array_reverse( $tags, true );
	elseif ( 'RAND' == $order ) {
		$keys = array_rand( $tags, count( $tags ) );
		foreach ( $keys as $key )
			$temp[$key] = $tags[$key];
		$tags = $temp;
		unset( $temp );
	}

	$counts = array();
	foreach ( (array) $tags as $key => $tag )
		$counts[ $key ] = $tag->count;

	$min_count = min( $counts );
	$spread = max( $counts ) - $min_count;
	if ( $spread <= 0 )
		$spread = 1;
	$em_step = $largest / $spread;

	$a = array();

	$rel = ( is_object( $wp_rewrite ) && $wp_rewrite->using_permalinks() ) ? ' rel="tag"' : '';
	$target_str = empty($target) ? '' : ' target="'.$target.'"';

	foreach ( $tags as $key => $tag ) {
		$count = $tag->count;
		$tag_link = '#' != $tag->link ? clean_url( $tag->link ) : '#';
		$tag_id = isset($tags[ $key ]->id) ? $tags[ $key ]->id : $key;
		$tag_name = $tags[ $key ]->name;
		$extra = $tags[ $key ]->extra;
		$em1 = '';
		$em2 = '';
		for ($i = 0; $i < ($count - $min_count) * $em_step; ++$i) {
			$em1 .= '<em>';
			$em2 .= '</em>';
		}
		$title = ' title="' . attribute_escape( $topic_count_text_callback( $count, $tag_name ) ) . '"';
		if ('nolink' == $format)
			$span = '<span'.$title.$rel.'>'.$tag_name.'</span>'.$extra;
		else
			$span = '<a href="'.$tag_link.'"'.$title.$rel.$target_str.'>'.$tag_name.'</a>'.$extra;
		$a[] = $em1 . $span . $em2;
	}

	switch ( $format ) :
	case 'array' :
		$return =& $a;
		break;
	case 'list' :
		$return = '<ul class="seo-tag-cloud">'."\n\t".'<li>';
		$return .= join( "</li>\n\t<li>", $a );
		$return .= "</li>\n</ul>\n";
		break;
	case 'nolink' :
	case 'flat' :
	default :
		$return = '<div class="seo-tag-cloud">'."\n\t";
		$return .= join( "\n", $a );
		$return .= '</div>'."\n";
		break;
	endswitch;

	return apply_filters( 'wp_generate_tag_cloud', $return, $tags, $args );
}

/**
 * Default text for tooltip for tag links
 *
 * @param integer $count number of posts with that tag
 * @return string text for the tooltip of a tag link.
 */
function seo_tag_cloud_default_topic_count_text( $count, $tag ) {
	return sprintf( __ngettext('%s: %s topic', '%s: %s topics', $count, 'seo-tag-cloud'), $tag, number_format_i18n( $count ) );
}

/**
 * Display tag cloud widget.
 *
 * @since 2.3.0
 *
 * @param array $args Widget arguments.
 */
function seo_tag_cloud_widget($args) {
	$defaults = array(
		'title' => __('Tag Cloud', 'seo-tag-cloud'),
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'topic_count_text_callback' => 'seo_tag_cloud_default_topic_count_text',
		'target' => '', 'em_step' => 1.0
	);
	$args = wp_parse_args( $args, $defaults );
	$options = get_option('widget_seo_tag_cloud');
	if (empty($options['title']))
		unset($options['title']);
	$args = wp_parse_args( $options, $args );
	extract($args);
	$title = apply_filters('widget_title', $title);

	echo $before_widget;
	echo $before_title . $title . $after_title;
	seo_tag_cloud( $args );
	if (!(bool)$options['hide-credit'])
		printf('<span class="credit">'.__('Powered by %s','seo-tag-cloud').'</span>', '<a href="http://blog.fleischer.hu/wordpress/seo-tag-cloud/" title="'.__('SEO Tag Cloud Widget Plugin for Wordpress','seo-tag-cloud').'">'.__('SEO Tag Cloud','seo-tag-cloud').'</a>');
	echo $after_widget;
}

/**
 * Manage WordPress Tag Cloud widget options.
 *
 * Displays management form for changing the tag cloud widget title.
 *
 * @since 2.3.0
 */
function seo_tag_cloud_widget_control() {
	$options = $newoptions = get_option('widget_seo_tag_cloud');

	if ( isset($_POST['seo-tag-cloud-submit']) ) {
		$newoptions['title'] = strip_tags(stripslashes($_POST['seo-tag-cloud-title']));
		$newoptions['number'] = strip_tags(stripslashes($_POST['seo-tag-cloud-number']));
		$newoptions['text-transform'] = strip_tags(stripslashes($_POST['seo-tag-cloud-text-transform']));
		$newoptions['hide-credit'] = strip_tags(stripslashes($_POST['seo-tag-cloud-hide-credit']));
		$newoptions['target'] = strip_tags(stripslashes($_POST['seo-tag-cloud-target']));
	}

	if ( $options != $newoptions ) {
		$options = $newoptions;
		update_option('widget_seo_tag_cloud', $options);
	}

	$title = attribute_escape( $options['title'] );
	$number = attribute_escape( $options['number'] );
	$text_transform = attribute_escape( $options['text-transform'] );
	$target = attribute_escape( $options['target'] );
	$hide_credit = (bool) $options['hide-credit'];
?>
	<p><label for="seo-tag-cloud-title">
	<?php _e('Title', 'seo-tag-cloud') ?>: <input type="text" class="widefat" id="seo-tag-cloud-title" name="seo-tag-cloud-title" value="<?php echo $title ?>" /></label>
	</p>
	<p><label for="seo-tag-cloud-number">
	<?php _e('Number of tags to show', 'seo-tag-cloud') ?>: <input type="text" class="widefat" style="width: 25px; text-align: center;" id="seo-tag-cloud-number" name="seo-tag-cloud-number" value="<?php echo $number ?>" /></label>
	</p>
	<p>
		<?php _e('Text transform', 'seo-tag-cloud') ?>:<br />
		<label for="seo-tag-cloud-text-transform-none" style="text-transform:none"><input type="radio" class="radio" id="seo-tag-cloud-text-transform-none" name="seo-tag-cloud-text-transform" value="none"<?php echo 'none' == $text_transform || '' == $text_transform ? ' checked="checked"' : '' ?> /> <?php _e('none', 'seo-tag-cloud') ?></label>
		<label for="seo-tag-cloud-text-transform-uppercase" style="text-transform:uppercase"><input type="radio" class="radio" id="seo-tag-cloud-text-transform-uppercase" name="seo-tag-cloud-text-transform" value="uppercase"<?php echo 'uppercase' == $text_transform ? ' checked="checked"' : '' ?> /> <?php _e('uppercase', 'seo-tag-cloud') ?></label>
		<label for="seo-tag-cloud-text-transform-capitalize" style="text-transform:capitalize"><input type="radio" class="radio" id="seo-tag-cloud-text-transform-capitalize" name="seo-tag-cloud-text-transform" value="capitalize"<?php echo 'capitalize' == $text_transform ? ' checked="checked"' : '' ?> /> <?php _e('capitalize', 'seo-tag-cloud') ?></label>
		<label for="seo-tag-cloud-text-transform-lowercase" style="text-transform:lowercase"><input type="radio" class="radio" id="seo-tag-cloud-text-transform-lowercase" name="seo-tag-cloud-text-transform" value="lowercase"<?php echo 'lowercase' == $text_transform ? ' checked="checked"' : '' ?> /> <?php _e('lowercase', 'seo-tag-cloud') ?></label>
	</p>
	<p><label for="seo-tag-cloud-target">
	<?php _e('Target for links', 'seo-tag-cloud') ?>: <input type="text" class="widefat" style="width: 100px;" id="seo-tag-cloud-target" name="seo-tag-cloud-target" value="<?php echo $target ?>" /></label>
	</p>
	<p><label for="seo-tag-cloud-hide-credit"><input id="seo-tag-cloud-hide-credit" name="seo-tag-cloud-hide-credit" type="checkbox" <?php checked( $hide_credit, true ); ?> class="checkbox" /> <?php _e('Hide credit','seo-tag-cloud'); ?></label></p>
	<p><a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&amp;hosted_button_id=10234211" target="_blank"><img src="https://www.paypal.com/en_US/i/btn/btn_donate_SM.gif" alt="<?_e('Donate')?>" /></a></p>
	<input type="hidden" name="seo-tag-cloud-submit" id="seo-tag-cloud-submit" value="1" />
<?php
}

function seo_tag_cloud_widget_style($args) {
	$defaults = array(
		'text-transform' => 'none',
	);
	$args = wp_parse_args( $args, $defaults );
	$options = get_option('widget_seo_tag_cloud');
	$options = wp_parse_args($options, $args);
	?>
<style type="text/css">
.seo-tag-cloud {font-size: 1.0em; text-transform: <?php echo !empty($options['text-transform']) ? $options['text-transform'] : 'none' ?>;}
.seo-tag-cloud li {display: inline;}
.seo-tag-cloud em {font-style: normal; font-size: 1.07em;}
.seo-tag-cloud div {padding: 10px;}
.credit {font-size: 50%;}
</style>
	<?php
}

/**
 * Register all of the default WordPress widgets on startup.
 *
 * Calls 'widgets_init' action after all of the WordPress widgets have been
 * registered.
 *
 * @since 2.2.0
 */
function seo_tag_cloud_widget_register() {
	if ( !is_blog_installed() )
		return;
	$widget_ops = array('classname' => 'seo_tag_cloud_widget', 'description' => __( "Your most used tags in SEO-friendly cloud format", 'seo-tag-cloud') );
	wp_register_sidebar_widget('seo_tag_cloud', __('SEO Tag Cloud', 'seo-tag-cloud'), 'seo_tag_cloud_widget', $widget_ops);
	wp_register_widget_control('seo_tag_cloud', __('SEO Tag Cloud', 'seo-tag-cloud'), 'seo_tag_cloud_widget_control' );
	if ( is_active_widget('seo_tag_cloud_widget') )
		add_action('wp_head', 'seo_tag_cloud_widget_style');
}

add_action('init', 'seo_tag_cloud_widget_register', 1);
?>